/******************************************************************************
flt_PNG.cpp : Defines the entry point for the DLL application.

Copyright  John Paul Chacha. All Rights Reserved.

This source code file, which has been provided by John Paul Chacha as part of
his software product for use only by licensed users of that product, includes
proprietary information of John Paul Chacha.

USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS OF THE LICENSE
AGREEMENT FURNISHED WITH THE PRODUCT.

IN PARTICULAR, JOHN PAUL CHACHA SHALL BE FREE FROM ANY CLAIMS OR LIABILITIES
ARISING OUT OF THE USE OR MISUSE OF THIS FILE AND/OR ITS CONTENTS.
******************************************************************************/
#include "../common/system.h"
#include "../common/plugin.h"
#include "../common/suite_info.h"
#include "../common/win_util.h"
#include "../common/str_util.h"
#include "../common/lang_util.h"
#include "resource.h"

#pragma warning(push)
	#include "../lib/libpng/png.h"
	#include "../lib/libpng/pngstruct.h"
	#include "../lib/zlib/zlib.h"
#pragma warning(pop)


//=============================================================================
#define PLUGIN_NAME			L"PNG File Support"
#define PLUGIN_AUTHOR		SUITE_CREATOR
#define PLUGIN_VERSION		SUITE_VER_INT32

#define MY_STORE_NAME		L"flt_PNG.dll"
#define MY_STORE_UUID		(*((LPDWORD)(".PNG")))

//-----------------------------------------------------------------------------
#define SETTINGS_VERSION	((1<<16)|(3)) //Oct 2019
struct my_plg_config
{
	unsigned long	uuid;
	unsigned long	version;
	bool			always_apply_bKGD;
	bool			dont_try_8bpp_PNG;
};

//-----------------------------------------------------------------------------
HINSTANCE api_inst;
my_plg_config settings={0,};

//-----------------------------------------------------------------------------
INT CALLBACK msg_dlg_about (HWND hDlg,UINT msg,WPARAM wpar,LPARAM lpar);
INT CALLBACK msg_dlg_config(HWND hDlg,UINT msg,WPARAM wpar,LPARAM lpar);

//=============================================================================
//=============================================================================



/******************************************************************************
WINDOWS (Win32) ENTRY POINT FOR THE DYNAMIC-LINK LIBRARY
******************************************************************************/
BOOL APIENTRY DllMain(
	HANDLE hModule,DWORD ul_reason_for_call,LPVOID lpReserved)
{
	//-------------------------------------------------------------------------
	switch(ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		api_inst=(HINSTANCE)hModule;
		break;
	case DLL_THREAD_ATTACH:
		break;
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		break;
	}
	//-------------------------------------------------------------------------
	UNREFERENCED_PARAMETER(lpReserved);
	return TRUE;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
Load plug-in settings
******************************************************************************/
int my_settings_load(pi_STATESTORE* pi_StateStore)
{
	int loaded;
	my_plg_config tmp;

	memset(&settings,0,sizeof(my_plg_config));
	settings.always_apply_bKGD=false;
	settings.dont_try_8bpp_PNG=false;

	loaded=0;
	if(pi_StateStore)
	{
		if(pi_StateStore->version>=PI_STATESTORE_VERSION)
		{
			if(pi_StateStore->perm_read(
				MY_STORE_NAME,&tmp,sizeof(my_plg_config)))
			{
				if((tmp.uuid==MY_STORE_UUID)&&(tmp.version==SETTINGS_VERSION))
				{
					memcpy(&settings,&tmp,sizeof(my_plg_config));
					loaded=1;
				}
			}
		}
	}
	return loaded;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
Save plug-in settings
******************************************************************************/
int my_settings_save(pi_STATESTORE* pi_StateStore)
{
	if(!pi_StateStore)
	{
		return 0;
	}
	settings.uuid=MY_STORE_UUID;
	settings.version=SETTINGS_VERSION;
	return pi_StateStore->perm_write(
		MY_STORE_NAME,&settings,sizeof(my_plg_config));
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This function is called to query plugin's name and type
******************************************************************************/
int plg_GetInfo(plg_INFO* info)
{
	info->api_type=PLUGIN_APITYPE_FILE;
	info->api_version=PLUGIN_INTERFACE_VERSION;
	info->plg_version=PLUGIN_VERSION;
	StringCchCopy(info->plg_name  ,_countof(info->plg_name)  ,PLUGIN_NAME  );
	StringCchCopy(info->plg_author,_countof(info->plg_author),PLUGIN_AUTHOR);
	return 1;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This function is called to display plugin dialog boxes
******************************************************************************/
int plg_ShowDialog(plg_DIALOG* data)
{
	int i;

	if(!data)
	{
		return PLUGIN_ERR_BAD_PARAM;
	}
	if(data->pi_BasicUtils)
	{
		if(data->pi_BasicUtils->version>=PI_BASICUTILS_VERSION)
		{
			if((data->hwnd)&&(!(lang_IsConnected())))
			{
				lang_Connect(data->pi_BasicUtils->lang_TranslateCore);
			}
		}
	}

	//-------------------------------------------------------------------------
	my_settings_load(data->pi_StateStore);

	switch(data->dialog_id)
	{
	case PLG_DIALOG_ABOUT:
		DialogBoxParam(api_inst,(LPCTSTR)IDD_ABOUT,data->hwnd,msg_dlg_about,0);
		return PLUGIN_OKAY;
	case PLG_DIALOG_CONFIG:
		i=DialogBoxParam(api_inst,(LPCTSTR)IDD_CONFIG,data->hwnd,
			msg_dlg_config,0);
		if(i>0)
		{
			if(!(my_settings_save(data->pi_StateStore)))
			{
				lang_MessageBox(data->hwnd,
					L"Error saving configuration data",
					L"Internal Error",MB_OK|MB_ICONERROR);
				break;
			}
		}
		return 1;
	case PLG_DIALOG_HELP:
		return PLUGIN_ERR_NO_SUPPORT;
	}
	return PLUGIN_ERR_BAD_PARAM;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
I'm sure the nice fellows at ->PNG had a good reason for not
using the time-tested 'magic-number' method.
******************************************************************************/
#ifndef png_jmpbuf
#  define png_jmpbuf(png_ptr) ((png_ptr)->jmpbuf)
#endif
#define ERROR_LIST_LEN 1024

void my_png_error(png_structp ppng, png_const_charp error)
{
	wchar_t werror[256];
	mbstowcs(werror,error,256);
	StringCchCat((wchar_t*)ppng->error_ptr,ERROR_LIST_LEN,werror);
	StringCchCat((wchar_t*)ppng->error_ptr,ERROR_LIST_LEN,L"\r\n");
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This is the plugin DLL function that Chasys Draw IES calls to check the load
operation support level for a specified file extension
******************************************************************************/
int flt_CheckLoad(flt_CHECK_L* info)
{
	if(_wcsicmp(info->extension,L".png")==0)
	{
		info->level_image=255;
		info->level_layer=255;
		info->level_other=255;
		info->support_flags=0;
		return 1;
	}
	if((info->hdr_data)&&(info->hdr_size>8))
	{
		BYTE png_signature[8]={137,80,78,71,13,10,26,10};
		if(memcmp(info->hdr_data,png_signature,8)==0)
		{
			info->level_image=255;
			info->level_layer=255;
			info->level_other=255;
			info->support_flags=0;
			return 1;
		}
	}
	return 0;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This is the plugin DLL function that Chasys Draw IES calls to check the save
operation support level for a specified file extension
******************************************************************************/
int flt_CheckSave(flt_CHECK_S* info)
{
	if(_wcsicmp(info->extension,L".png")==0)
	{
		info->level_image=255;
		info->level_layer=255;
		info->level_other=255;
		info->support_flags=0;
		return 1;
	}
	return 0;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This is the plugin DLL function that Chasys Draw IES calls to load images
******************************************************************************/
int flt_LoadImage(flt_IMAGE_L* data)
{
	int bit_depth,color_type,interlace_type,last_error;
	unsigned int row,sig_read,ppi_x,ppi_y;
	wchar_t error_list[ERROR_LIST_LEN];
	png_uint_32 width,height;
	png_color_16 my_bKGD;
	png_color_16 *img_bKGD;
	png_structp png_ptr;
	png_infop info_ptr;
	png_bytep *row_ptrs;
	FILE *fp;

	//-------------------------------------------------------------------------
	if(!data)
	{
		return PLUGIN_ERR_BAD_PARAM;
	}
	if(!data->pi_BasicImage)
	{
		return PLUGIN_ERR_BAD_HOST;
	}
	if(data->pi_BasicImage->version<PI_BASICIMAGE_VERSION)
	{
		return PLUGIN_ERR_BAD_HOST;
	}
	if(data->pi_BasicUtils)
	{
		if(data->pi_BasicUtils->version>=PI_BASICUTILS_VERSION)
		{
			if((data->hwnd)&&(!(lang_IsConnected())))
			{
				lang_Connect(data->pi_BasicUtils->lang_TranslateCore);
			}
		}
	}

	//-------------------------------------------------------------------------
	my_settings_load(data->pi_StateStore);
	sig_read=0;

	//-------------------------------------------------------------------------
	//create and initialize library structures
	png_ptr=png_create_read_struct(PNG_LIBPNG_VER_STRING,0,0,0);
	if(!png_ptr)
	{
		return PLUGIN_ERR_INTERNAL_U;
	}

	error_list[0]=0;
	png_ptr->error_fn=my_png_error;
	png_ptr->error_ptr=error_list;
	info_ptr=png_create_info_struct(png_ptr);
	if(!info_ptr)
	{
		png_destroy_read_struct(&png_ptr,&info_ptr,NULL);
		return PLUGIN_ERR_INTERNAL_U;
	}

	last_error=PLUGIN_ERR_INTERNAL_U;
	if(setjmp(png_jmpbuf(png_ptr)))
	{
		goto err_internal;
	}

	//-------------------------------------------------------------------------
	//open the file and read the header
	fp=_wfopen(data->file,L"rb");
	if(!fp)
	{
		png_destroy_read_struct(&png_ptr,&info_ptr,NULL);
		return PLUGIN_ERR_FILE_OPEN_R;
	}

	last_error=PLUGIN_ERR_NO_SUPPORT;
	png_init_io(png_ptr,fp);
	png_set_sig_bytes(png_ptr,sig_read);
	png_read_info(png_ptr,info_ptr);
	png_get_IHDR(png_ptr,info_ptr,&width,&height,&bit_depth,&color_type,
		&interlace_type,NULL,NULL);

	//-------------------------------------------------------------------------
	//set up stuff
	last_error=PLUGIN_ERR_INTERNAL_U;

	png_set_swap(png_ptr);//swap bytes of 16 bit files to small endian
#ifdef PNG_READ_SCALE_16_TO_8_SUPPORTED
	png_set_scale_16(png_ptr);
#else
	png_set_strip_16(png_ptr);
#endif

	png_set_packing(png_ptr);//expand bpp=1/2/4 to full bytes
	png_set_invert_mono(png_ptr);
	switch(color_type)
	{
	case PNG_COLOR_TYPE_PALETTE:
		png_set_palette_to_rgb(png_ptr);
		break;
	case PNG_COLOR_TYPE_GRAY:
	case PNG_COLOR_TYPE_GRAY_ALPHA:
		if(bit_depth<8)
		{
			png_set_expand_gray_1_2_4_to_8(png_ptr);
		}
		png_set_gray_to_rgb(png_ptr);
		break;
	}
	if(png_get_valid(png_ptr,info_ptr,PNG_INFO_tRNS))
	{
		png_set_tRNS_to_alpha(png_ptr);
	}
	if(color_type&PNG_COLOR_MASK_COLOR)
	{
		png_set_bgr(png_ptr);//flip RGB/RGBA to BGR/BGRA
	}
	png_set_invert_alpha(png_ptr);
	png_set_filler(png_ptr,0,PNG_FILLER_AFTER);//filler for non-alpha files

	//background color
	if( (settings.always_apply_bKGD)&&
		(png_get_bKGD(png_ptr,info_ptr,&img_bKGD)))
	{
		png_set_background(png_ptr,img_bKGD,PNG_BACKGROUND_GAMMA_FILE,1,1);
	}
	else
	{
		my_bKGD.red  =0xffff;
		my_bKGD.green=0xffff;
		my_bKGD.blue =0xffff;
		my_bKGD.gray =0xffff;
		my_bKGD.index=0;
		png_set_background(png_ptr,&my_bKGD,PNG_BACKGROUND_GAMMA_UNKNOWN,0,1);
	}

	//-------------------------------------------------------------------------
	//optional call to gamma correct and add the background to the palette
	//and update info structure
	png_read_update_info(png_ptr,info_ptr);

	//-------------------------------------------------------------------------
	//create layer
	last_error=PLUGIN_ERR_INTERNAL_U;

	data->width =width;
	data->height=height;
	data->meta_data=0;
	data->host_flags=0;
	if(data->create(data)==0)
	{
		last_error=min(0,data->host_response);
		goto err_internal;
	}

	//insert MARK attachment
	if(png_get_valid(png_ptr,info_ptr,PNG_INFO_pHYs))
	{
		if((data->meta_write)&&(data->pi_MetaMarkup))
		{
			char markup[3072];
			markup[0]=0;
			ppi_x=png_get_x_pixels_per_inch(png_ptr,info_ptr);
			ppi_y=png_get_y_pixels_per_inch(png_ptr,info_ptr);
			if((ppi_x>0)&&(ppi_y>0))
			{
				data->pi_MetaMarkup->ppi_set(markup,3072,ppi_x,ppi_y);
				data->meta_data=(unsigned char*)markup;
				data->meta_write(data,"MARK",strlen(markup)+1);
			}
		}
	}

	//-------------------------------------------------------------------------
	//read the image
	row_ptrs=(png_bytep*)GlobalAlloc(GPTR,(height+16)*sizeof(png_bytep));
	if(!row_ptrs)
	{
		png_destroy_read_struct(&png_ptr,&info_ptr,NULL);
		fclose(fp);
		return PLUGIN_ERR_OUT_OF_MEM;
	}
	for(row=0;row<height;row++)
	{
		row_ptrs[row]=(unsigned char*)(data->lp_pix+(row*data->pitch));
	}
	png_read_image(png_ptr,row_ptrs);
	GlobalFree(row_ptrs);

	//read rest of file, and get additional chunks in info_ptr
	png_read_end(png_ptr,info_ptr);

	//clean up after the read, and free any memory allocated
	png_destroy_read_struct(&png_ptr,&info_ptr,NULL);

	//-------------------------------------------------------------------------
	//close the file
	fclose(fp);
	data->img_type=PLUGIN_IMGTYPE_COMPOSITE;
	data->img_subtype=PLUGIN_IMGSUBTYPE_NORMAL;
	return 1;

	//=========================================================================
err_internal:
	png_destroy_read_struct(&png_ptr,&info_ptr,NULL);
	fclose(fp);
	if(wcslen(error_list))
	{
		if(data->set_err_info)
		{
			data->set_err_info(data,error_list);
		}
	}
	return last_error;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This is the plugin DLL function that Chasys Draw IES calls to save images
******************************************************************************/
int flt_SaveImage(flt_IMAGE_S* data)
{
	unsigned int xx,yy,len,ref,flag,use_palette,pal_size;
	unsigned long palette[PNG_MAX_PALETTE_LENGTH];
	png_color png_palette[PNG_MAX_PALETTE_LENGTH];
	unsigned long ul,*p_mem,*p_src,*p_dst;
	wchar_t error_list[ERROR_LIST_LEN];
	double ppi_x,ppi_y;
	char* markup;
	png_infop info_ptr;
	png_structp png_ptr;
	png_bytep *row_ptrs;
	FILE *fp;

	//-------------------------------------------------------------------------
	if(!data)
	{
		return PLUGIN_ERR_BAD_PARAM;
	}
	if(!data->pi_BasicImage)
	{
		return PLUGIN_ERR_BAD_HOST;
	}
	if(data->pi_BasicImage->version<PI_BASICIMAGE_VERSION)
	{
		return PLUGIN_ERR_BAD_HOST;
	}
	if(data->pi_BasicUtils)
	{
		if(data->pi_BasicUtils->version>=PI_BASICUTILS_VERSION)
		{
			if((data->hwnd)&&(!(lang_IsConnected())))
			{
				lang_Connect(data->pi_BasicUtils->lang_TranslateCore);
			}
		}
	}

	//-------------------------------------------------------------------------
	if(data->layer_count>1)
	{
		return PLUGIN_ERR_NOT_FLAT;
	}
	my_settings_load(data->pi_StateStore);
	p_mem=0;

	//-------------------------------------------------------------------------
	//open the file stream for binary writing
	fp=_wfopen(data->file,L"wb");
	if(!fp)
	{
		return PLUGIN_ERR_FILE_OPEN_W;
	}

	//-------------------------------------------------------------------------
	//create and initialize library structures
	png_ptr=png_create_write_struct(PNG_LIBPNG_VER_STRING,0,0,0);
	if(!png_ptr)
	{
		fclose(fp);
		return PLUGIN_ERR_INTERNAL_U;
	}

	error_list[0]=0;
	png_ptr->error_fn=my_png_error;
	png_ptr->error_ptr=error_list;
	info_ptr=png_create_info_struct(png_ptr);
	if(!info_ptr)
	{
		fclose(fp);
		png_destroy_write_struct(&png_ptr,NULL);
		return PLUGIN_ERR_INTERNAL_U;
	}

	if(setjmp(png_jmpbuf(png_ptr)))
	{
		goto err_internal;
	}

	//-------------------------------------------------------------------------
	//try building a palette for 8-bpp PNG
	if(settings.dont_try_8bpp_PNG)
	{
		use_palette=false;
		pal_size=0;
	}
	else
	{
		use_palette=true;
		pal_size=0;
		for(yy=0;yy<data->height;yy++)
		{
			if(!use_palette)
			{
				break;
			}
			p_src=data->lp_pix+(data->pitch*yy);
			for(xx=0;xx<data->width;xx++)
			{
				if((p_src[xx]>>24)>0) // we have alpha data
				{
					use_palette=false;
					break;
				}
				flag=0;
				for(ref=0;((ref<pal_size)&&(!flag));ref++)
				{
					flag|=(palette[ref]==p_src[xx]);
				}
				if(flag)
				{
					continue;
				}
				if(pal_size>=(PNG_MAX_PALETTE_LENGTH-1))
				{
					use_palette=false;
					break;
				}
				palette[pal_size]=p_src[xx];
				pal_size+=1;
			}
		}
		if(!use_palette)
		{
			pal_size=0;
		}
	}

	//-------------------------------------------------------------------------
	//get the bitmap data, load it into buffer
	ul=data->width*data->height*4;
	p_mem=(unsigned long*)GlobalAlloc(GMEM_FIXED,ul);
	if(!p_mem)
	{
		png_destroy_write_struct(&png_ptr,&info_ptr);
		fclose(fp);
		return PLUGIN_ERR_OUT_OF_MEM;
	}
	row_ptrs=(png_bytep*)GlobalAlloc(GPTR,(data->height+16)*sizeof(png_bytep));
	if(!row_ptrs)
	{
		png_destroy_write_struct(&png_ptr,&info_ptr);
		fclose(fp);
		return PLUGIN_ERR_OUT_OF_MEM;
	}
	for(yy=0;yy<data->height;yy++)
	{
		p_src=data->lp_pix+(data->pitch*yy);
		p_dst=p_mem+(data->width*yy);
		if(use_palette)
		{
			for(xx=0;xx<data->width;xx++)
			{
				for(ref=0;ref<pal_size;ref++)
				{
					if(palette[ref]==p_src[xx])
					{
						((unsigned char*)p_dst)[xx]=(unsigned char)ref;
						break;
					}
				}
			}
		}
		else
		{
			for(xx=0;xx<data->width;xx++)
			{
				p_dst[xx]=
					((p_src[xx]>>16)&0x000000ff)|
					( p_src[xx]     &0x0000ff00)|
					((p_src[xx]<<16)&0x00ff0000)|
					(( ~(p_src[xx]))&0xff000000);
			}
		}
		row_ptrs[yy]=(unsigned char*)(&(p_mem[(yy*data->width)]));
	}

	//-------------------------------------------------------------------------
	//start file output, initialize image information
	png_init_io(png_ptr,fp);
	png_set_IHDR(png_ptr,info_ptr,data->width,data->height,8,
		((use_palette)? (PNG_COLOR_TYPE_PALETTE):(PNG_COLOR_TYPE_RGBA)),
		PNG_INTERLACE_NONE,PNG_COMPRESSION_TYPE_BASE,PNG_FILTER_TYPE_BASE);

	if(use_palette)
	{
		for(ref=0;ref<pal_size;ref++)
		{
			png_palette[ref].red  =(png_byte)((palette[ref]>>16)&0xff);
			png_palette[ref].green=(png_byte)((palette[ref]>> 8)&0xff);
			png_palette[ref].blue =(png_byte)((palette[ref]    )&0xff);
		}
		png_set_PLTE(png_ptr,info_ptr,png_palette,PNG_MAX_PALETTE_LENGTH);
	}

	if((data->meta_read)&&(data->pi_MetaMarkup))
	{
		len=data->meta_read(data,"MARK");
		if(len)
		{
			markup=(char*)data->meta_data;
			markup[len-1]=0;
			if(data->pi_MetaMarkup->ppi_get(markup,&ppi_x,&ppi_y))
			{
				xx=(int)((ppi_x/0.0254)+0.5);
				yy=(int)((ppi_y/0.0254)+0.5);
				png_set_pHYs(png_ptr,info_ptr,xx,yy,PNG_RESOLUTION_METER);
			}
		}
	}

	//png_set_gAMA(png_ptr,info_ptr,gamma);

	png_text text_ptr[1];
	char key0[]="Author";
	char text0[]=SUITE_FULLNAME_A;
	text_ptr[0].key=key0;
	text_ptr[0].text=text0;
	text_ptr[0].compression=PNG_TEXT_COMPRESSION_NONE;
	text_ptr[0].itxt_length=0;
	text_ptr[0].lang=NULL;
	text_ptr[0].lang_key=NULL;
	png_set_text(png_ptr,info_ptr,text_ptr,1);

	//-------------------------------------------------------------------------
	//write the file header
	png_write_info(png_ptr,info_ptr);

	//-------------------------------------------------------------------------
	//write the image data
	//png_set_bgr(png_ptr);//flip BGR pixels to RGB
	png_write_image(png_ptr,row_ptrs);
	png_write_end(png_ptr,info_ptr);
	GlobalFree(row_ptrs);

	//-------------------------------------------------------------------------
	//clean up, free any memory,close the file
	if(p_mem)
	{
		GlobalFree(p_mem);
		p_mem=0;
	}
	png_destroy_write_struct(&png_ptr,&info_ptr);
	fclose(fp);
	return 1;

	//=========================================================================
err_internal:
	if(p_mem)
	{
		GlobalFree(p_mem);
		p_mem=0;
	}
	png_destroy_write_struct(&png_ptr,&info_ptr);
	fclose(fp);
	if(wcslen(error_list))
	{
		if(data->set_err_info)
		{
			data->set_err_info(data,error_list);
		}
	}
	return PLUGIN_ERR_INTERNAL_U;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This function processes messages for about dialog
******************************************************************************/
INT CALLBACK msg_dlg_about(HWND hDlg,UINT msg,WPARAM wpar,LPARAM lpar)
{
	wchar_t* pstr;

	//-------------------------------------------------------------------------
	switch(msg)
	{
	case WM_INITDIALOG:
		SetWindowText(hDlg,L"About " PLUGIN_NAME);
		lang_TranslateWindow(hDlg);
		pstr=string_ReadTextResource(api_inst,(LPCTSTR)IDR_README,L"TEXT");
		if(pstr)
		{
			string_SetNewline(pstr,GlobalSize(pstr)/sizeof(wchar_t),true);
			SetDlgItemText(hDlg,IDC_EDIT1,pstr);
			GlobalFree(pstr);
		}
		window_Center(hDlg,GetParent(hDlg));
		return 1|lpar;
	case WM_COMMAND:
		switch(LOWORD(wpar))
		{
		case IDOK    :
			EndDialog(hDlg,1);
			break;
		case IDCANCEL:
			EndDialog(hDlg,0);
			break;
		}
		return 1;
	}
	return 0;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


/******************************************************************************
This function processes messages for config dialog
******************************************************************************/
INT CALLBACK msg_dlg_config(HWND hDlg,UINT msg,WPARAM wpar,LPARAM lpar)
{
	wchar_t str[MAX_PATH];

	switch(msg)
	{
	case WM_INITDIALOG:
		window_Center(hDlg,GetParent(hDlg));
		lang_TranslateWindow(hDlg);
		lang_TranslateText(str,_countof(str),L"Configure");
		StringCchCat(str,_countof(str),L" " PLUGIN_NAME);
		SetWindowText(hDlg,str);

		CheckDlgButton(hDlg,IDC_CHECK1,settings.always_apply_bKGD);
		CheckDlgButton(hDlg,IDC_CHECK2,settings.dont_try_8bpp_PNG);
		return 1|lpar;
	case WM_COMMAND:
		switch(LOWORD(wpar))
		{
		case IDOK:
			settings.always_apply_bKGD=(IsDlgButtonChecked(hDlg,IDC_CHECK1)!=0);
			settings.dont_try_8bpp_PNG=(IsDlgButtonChecked(hDlg,IDC_CHECK2)!=0);
			EndDialog(hDlg,1);
			break;
		case IDCANCEL:
			EndDialog(hDlg,0);
			break;
		}
		return 1;
	}
	return 0;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
