/******************************************************************************
FastExternals Plug-in Interface Header

Copyright  John Paul Chacha. All Rights Reserved.

This source code file, which has been provided by John Paul Chacha as part of 
his software product for use only by licensed users of that product, includes 
proprietary information of John Paul Chacha.

USE OF THIS SOFTWARE IS GOVERNED BY THE TERMS AND CONDITIONS OF THE LICENSE 
AGREEMENT FURNISHED WITH THE PRODUCT.

IN PARTICULAR, JOHN PAUL CHACHA SHALL BE FREE FROM ANY CLAIMS OR LIABILITIES 
ARISING OUT OF THE USE OR MISUSE OF THIS FILE AND/OR ITS CONTENTS.
******************************************************************************/

#pragma once

#include "../common/imagedefs.h"
#include "../common/layerdefs.h"
#include "../common/metadefs.h"


//=============================================================================



/******************************************************************************
*******************************************************************************
*******************************************************************************

                                GLOBAL DEFINES

*******************************************************************************
*******************************************************************************
******************************************************************************/



//=============================================================================

#define PLUGIN_INTERFACE_NAME			L"FastExternals"
#define PLUGIN_INTERFACE_VERSION$		L"3 rev 40"
#define PLUGIN_INTERFACE_VERSION		((3<<16)+40) // 19-Aug-2020
#define PLUGIN_INTERFACE_MIN_VER		((3<<16)+40)

#define PLUGIN_APITYPE_INVALID			0x00000000
#define PLUGIN_APITYPE_FILE				0x00000001
#define PLUGIN_APITYPE_DEVICE			0x00000002
#define PLUGIN_APITYPE_EFFECT			0x00000004
#define PLUGIN_APITYPE_ENGINE			0x00000100

#define PLUGIN_IMGTYPE_COMPOSITE		0x0000
#define PLUGIN_IMGTYPE_IMAGELIST		0x0001
#define PLUGIN_IMGTYPE_FRAMEANIM		0x0002
#define PLUGIN_IMGTYPE_OBJECTANIM		0x0003

#define PLUGIN_IMGSUBTYPE_NORMAL		0x0000
#define PLUGIN_IMGSUBTYPE_CLIPPED		0x0001
#define PLUGIN_IMGSUBTYPE_MULTIRES		0x0001
#define PLUGIN_IMGSUBTYPE_ONIONSKIN		0x0001

#define PLUGIN_HOSTFLAG_RESIZE			0x0001
#define PLUGIN_HOSTFLAG_NOSAVE			0x0002

#define PLUGIN_WARN_LAYER_COUNT			0x01 //layers exceed allowed maximum 
#define PLUGIN_WARN_LAYER_TYPE			0x02 //bad or unknown layer type found 
#define PLUGIN_WARN_BLEND_MODE			0x04 //bad or unknown blend mode found 
#define PLUGIN_WARN_IMAGE_DATA			0x08 //bad or unknown image data found 

//-----------------------------------------------------------------------------
#define PLUGIN_MAX_SURF_WIDTH			(64*1024)
#define PLUGIN_MAX_LAYER_COUNT			1024

//-----------------------------------------------------------------------------
#define PLUGIN_OKAY						  1 //success

#define PLUGIN_ERR_FAILED				  0 //general failure

#define PLUGIN_ERR_NO_SUPPORT			 -1 //not supported
#define PLUGIN_ERR_NO_FEATURE			 -2 //feature not implemented
#define PLUGIN_ERR_NO_IMAGE				 -3 //file doesn't contain image
#define PLUGIN_ERR_COMPONENT			 -4 //component not found
#define PLUGIN_ERR_SYS_REQ				 -5 //system requirements
#define PLUGIN_ERR_DEVICE				 -6 //I/O device error

#define PLUGIN_ERR_BAD_HOST				-10 //host service not found
#define PLUGIN_ERR_BAD_PARAM			-11 //host parameter invalid
#define PLUGIN_ERR_BAD_IMAGE			-12 //invalid image
#define PLUGIN_ERR_BAD_WIDTH			-13 //unsupported image size
#define PLUGIN_ERR_BAD_MODE				-14 //unsupported image mode
#define PLUGIN_ERR_NOT_FLAT				-15 //image not flat (has layers)

#define PLUGIN_ERR_OUT_OF_MEM			-20 //out of memory
#define PLUGIN_ERR_BITMAP_ERR			-25 //bitmap error, unknown
#define PLUGIN_ERR_BITMAP_NUL			-26 //bitmap error, empty
#define PLUGIN_ERR_BITMAP_BIG			-27 //bitmap error, too big

#define PLUGIN_ERR_INTERNAL_U 			-30 //internal error, unspecified
#define PLUGIN_ERR_INTERNAL_1 			-31 //internal error, type #1
#define PLUGIN_ERR_INTERNAL_2 			-32 //internal error, type #2
#define PLUGIN_ERR_INTERNAL_3 			-33 //internal error, type #3
#define PLUGIN_ERR_INTERNAL_4 			-34 //internal error, type #4

#define PLUGIN_ERR_FILE_OPEN_R			-40 //cannot open for read
#define PLUGIN_ERR_FILE_OPEN_W			-41 //cannot open for write
#define PLUGIN_ERR_FILE_READ			-42 //error reading data
#define PLUGIN_ERR_FILE_WRITE			-43 //error writing data
#define PLUGIN_ERR_FILE_CORRUPT			-44 //invalid data in file

#define PLUGIN_ERR_TIMEOUT				-98 //cancelled by timeout
#define PLUGIN_ERR_USER_CANCEL			-99 //cancelled by the user

//-----------------------------------------------------------------------------
#define PLG_DIALOG_ABOUT				0x00000000
#define PLG_DIALOG_CONFIG				0x00000001
#define PLG_DIALOG_HELP 				0x00000002

//-----------------------------------------------------------------------------
#define PLG_FLT_XFORM_ROTATE_090		0x00000001
#define PLG_FLT_XFORM_ROTATE_180		0x00000002
#define PLG_FLT_XFORM_ROTATE_270		0x00000003
#define PLG_FLT_XFORM_FLIP_H			0x00000004
#define PLG_FLT_XFORM_FLIP_V			0x00000005
#define PLG_FLT_XFORM_TRANSPOSE			0x00000006
#define PLG_FLT_XFORM_TRANSVERSE		0x00000007

#define PLG_EFX_FLAGS_HIDE_GUI			0x00000001


//=============================================================================



/******************************************************************************
*******************************************************************************
*******************************************************************************

                                HELPER MACROS

*******************************************************************************
*******************************************************************************
******************************************************************************/



//=============================================================================

//-----------------------------------------------------------------------------
//Macro for converting pixels between ARGB and ABGR
#define plg_swap_rgb(c) \
	((c&0xff00ff00)|((c>>16)&0x000000ff)|((c<<16)&0x00ff0000))

//-----------------------------------------------------------------------------
//Macro for restricting a value to a given set of limits
#define plg_limit_value(a,lo,hi) \
	{ if((a)<(lo)){a=(lo);} else if((a)>(hi)){a=(hi);} }

//-----------------------------------------------------------------------------
//Macro for checking a value against a given set of limits
#define plg_not_in_range(a,lo,hi) ((a<(lo))||(a>(hi)))


//=============================================================================



/******************************************************************************
*******************************************************************************
*******************************************************************************

               CALLBACK SUITE STRUCTS FOR "BASIC PI" INTERFACES 

*******************************************************************************
*******************************************************************************
******************************************************************************/



//=============================================================================
	
//-----------------------------------------------------------------------------
//pi_StateStore
//-----------------------------------------------------------------------------
#define PI_STATESTORE_VERSION					0x00020001

#pragma pack(push)
#pragma pack(1)
	struct pi_STATESTORE
	{
		unsigned long	version;
		unsigned long	unused_1;
		int (__cdecl*	temp_read )(wchar_t* name,void* data,int size);
		int (__cdecl*	temp_write)(wchar_t* name,void* data,int size);
		int (__cdecl*	temp_size )(wchar_t* name);
		int (__cdecl*	perm_read )(wchar_t* name,void* data,int size);
		int (__cdecl*	perm_write)(wchar_t* name,void* data,int size);
		int (__cdecl*	perm_size )(wchar_t* name);
	//32
		unsigned char	reserved_x[1024-32];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//pi_BasicQuery
//-----------------------------------------------------------------------------
#define PI_BASICQUERY_VERSION					0x00010002

#pragma pack(push)
#pragma pack(1)
	struct pi_BASICQUERY
	{
		unsigned long	version;
		unsigned long	unused_1;
		void* (__cdecl* get_PI)(wchar_t* name);
		int   (__cdecl* get_data)(wchar_t* name,void* data,int size);
		int   (__cdecl* set_data)(wchar_t* name,void* data,int size);
	//20
		unsigned char reserved_x[1024-20];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//pi_BasicImage
//-----------------------------------------------------------------------------
#define PI_BASICIMAGE_VERSION					0x0001000d // 16 Dec 2020

#pragma pack(push)
#pragma pack(1)
	struct pibi_IMAGE
	{
		unsigned long	width;
		unsigned long	height;
		unsigned long	pitch;
		unsigned long	unused;
		unsigned long	*p_pix;
		void			*reserved;
	};
	struct pibi_RECT
	{
		long	x0;
		long	y0;
		long	x1;
		long	y1;
		bool	valid;
	};
	struct pibi_FILEINFO
	{
		HWND			hwnd;
		wchar_t*		file;
		void*			memfile_data;
		unsigned long	memfile_size;
		wchar_t*		memfile_type_hint;
		bool			get_metadata;
		void*			metadata_ptr;
		unsigned long	metadata_size;
	};
#pragma pack(pop)

#pragma pack(push)
#pragma pack(1)
	struct pi_BASICIMAGE
	{
		unsigned long	version;
		unsigned long	unused_1;
	//8
		int (__cdecl*	create)(pibi_IMAGE* img,int width,int height);
		int (__cdecl*	virtualize)(pibi_IMAGE* img,int width,int height,
							int pitch,unsigned long* p_pix);
		int (__cdecl*	destroy)(pibi_IMAGE* img);
		int (__cdecl*	resize)(pibi_IMAGE* img,int width,int height);
	//24
		int (__cdecl*	lock)(pibi_IMAGE* img);
		int (__cdecl*	unlock)(pibi_IMAGE* img);
	//32
		HDC (__cdecl*	gdi_getDC)(pibi_IMAGE* img);
		int (__cdecl*	gdi_freeDC)(pibi_IMAGE* img,HDC hdc);
		HBITMAP (__cdecl* gdi_makeHBMP)(pibi_IMAGE* img);
		void*			reserved_1;
	//48
		int (__cdecl*	blit)(
							pibi_IMAGE* dst,int dx0,int dy0,int dx1,int dy1,
							pibi_IMAGE* src,int sx0,int sy0,int sx1,int sy1,
							unsigned long options,unsigned long param);
		int (__cdecl*	fill)(
							pibi_IMAGE* img,int x0,int y0,int x1,int y1,
							unsigned long color1,unsigned long color2,
							unsigned long color3,unsigned long color4,
							unsigned long options,int param1,int param2);
		int (__cdecl*	blit_DC)(
							HDC hdc,int dx0,int dy0,int dx1,int dy1,
							pibi_IMAGE* src,int sx0,int sy0,int sx1,int sy1,
							unsigned long options,unsigned long param);
		int (__cdecl*	quad_blit)(
							pibi_IMAGE* dst,
							int x0,int y0,int x1,int y1,int x2,int y2,int x3,int y3,
							pibi_IMAGE* src,int sx0,int sy0,int sx1,int sy1,
							unsigned long options,unsigned long param);
	//64
		int (__cdecl*	translate)(
							pibi_IMAGE* img,int x0,int y0,int x1,int y1,
							unsigned char *lut_r,unsigned char *lut_g,
							unsigned char *lut_b,unsigned char *lut_a,
							unsigned long options,void* param);
		int (__cdecl*	transform)(
							pibi_IMAGE* img,int x0,int y0,int x1,int y1,
							double *matrix_5x4,unsigned long options);
		int (__cdecl*	convolute)(
							pibi_IMAGE* dst,int x0,int y0,int x1,int y1,
							pibi_IMAGE* src,
							double *matrix,int mat_cols,int mat_rows,
							unsigned long mask,unsigned long options);
		int (__cdecl*	compare)(pibi_IMAGE* i_new,pibi_IMAGE* i_org,
							pibi_RECT *rt_diff);
	//80
		int (__cdecl*	load_file)(pibi_IMAGE* img,pibi_FILEINFO* info);
		int (__cdecl*	save_file)(pibi_IMAGE* img,pibi_FILEINFO* info);
		void*			reserved_3;
		void*			reserved_4;
	//96
		unsigned char	reserved_x[1024-96];//do not touch
	};
#pragma pack(pop)

//flags for the "options" parameter of ::blit()
#define PI_BI_BLIT_BLOCKY						0x00000000
#define PI_BI_BLIT_STRETCH						0x00000001
#define PI_BI_BLIT_MAXQUALITY					0x00000002
#define PI_BI_BLIT_ALPHABLEND					0x00000010
#define PI_BI_BLIT_RASTEROP  					0x00000020
#define PI_BI_BLIT_TILE_xxx						0x00030000
	#define PI_BI_BLIT_TILE_NORMAL				0x00010000
	#define PI_BI_BLIT_TILE_INVERT				0x00020000

//flags for the "options" parameter of ::fill()
//also used for "param" for blit::PI_BI_BLIT_RASTEROP
#define PI_BI_FILL_SHAPE_xxx					0x0000000f
	#define PI_BI_FILL_RECTANGLE				0x00000000
	#define PI_BI_FILL_ELLIPSE					0x00000001
	#define PI_BI_FILL_ROUNDRECT				0x00000002
#define PI_BI_FILL_ALPHABLEND					0x00000010
#define PI_BI_FILL_GRADIENT						0x00000020
#define PI_BI_FILL_RASTEROP_xxx					0x00ff0000
	#define PI_BI_FILL_RASTEROP_NONE            0x00000000
	#define PI_BI_FILL_RASTEROP_OR              0x00010000
	#define PI_BI_FILL_RASTEROP_AND             0x00020000
	#define PI_BI_FILL_RASTEROP_XOR             0x00030000
	#define PI_BI_FILL_RASTEROP_ADD             0x00040000
	#define PI_BI_FILL_RASTEROP_SUB             0x00050000
	#define PI_BI_FILL_RASTEROP_INC             0x00060000
	#define PI_BI_FILL_RASTEROP_DEC             0x00070000
	#define PI_BI_FILL_RASTEROP_MUL             0x00080000
	#define PI_BI_FILL_RASTEROP_DIV             0x00090000
	#define PI_BI_FILL_RASTEROP_OFF             0x000a0000
	#define PI_BI_FILL_RASTEROP_MIN             0x000b0000
	#define PI_BI_FILL_RASTEROP_MAX             0x000c0000
	#define PI_BI_FILL_RASTEROP_DIFF            0x000d0000
	#define PI_BI_FILL_RASTEROP_OUT             0x000e0000
	#define PI_BI_FILL_RASTEROP_TEX1            0x000f0000
	#define PI_BI_FILL_RASTEROP_TEX2            0x00100000
	#define PI_BI_FILL_RASTEROP_SCRN            0x00110000
	#define PI_BI_FILL_RASTEROP_OVER            0x00120000
	#define PI_BI_FILL_RASTEROP_DODG            0x00130000
	#define PI_BI_FILL_RASTEROP_BURN            0x00140000
	#define PI_BI_FILL_RASTEROP_LUM             0x00150000
	#define PI_BI_FILL_RASTEROP_CLR             0x00160000
	#define PI_BI_FILL_RASTEROP_HUE             0x00170000
	#define PI_BI_FILL_RASTEROP_SAT             0x00180000
	#define PI_BI_FILL_RASTEROP_VAL             0x00190000
	#define PI_BI_FILL_RASTEROP_LIT             0x001a0000
	#define PI_BI_FILL_RASTEROP_MASK            0x007d0000



//-----------------------------------------------------------------------------
//pi_BasicColor
//-----------------------------------------------------------------------------
#define PI_BASICCOLOR_VERSION					0x00010003

#pragma pack(push)
#pragma pack(1)
	struct pi_BASICCOLOR
	{
		unsigned long	version;
		unsigned long	unused_1;
	//8
		int	(__cdecl*	convert)(unsigned long *p_pix,int count,
			unsigned long cp_in,unsigned long cp_out);
		int	(__cdecl*	read_rgb)(unsigned long* p_pix,unsigned char* p_rgb,
			int count,int bpp,unsigned long flags);
	//16
		unsigned char	reserved_x[1024-16];//do not touch
	};
#pragma pack(pop)

//flags for "cp_in" and "cp_out" in ::convert()
#define PI_BC_CONVERT_CS_RGB					0x00000000
#define PI_BC_CONVERT_CS_YUV					0x00000001
#define PI_BC_CONVERT_CS_CMY					0x00000002
#define PI_BC_CONVERT_CS_CMYK					0x00000003
#define PI_BC_CONVERT_CS_HSL					0x00000004
#define PI_BC_CONVERT_CS_HSV					0x00000005
#define PI_BC_CONVERT_CS_HHSL					0x00000006
#define PI_BC_CONVERT_CS_HHSV					0x00000007
#define PI_BC_CONVERT_CS_LAB					0x00000008
#define PI_BC_CONVERT_CS_xxxx					0x0000000f
#define PI_BC_CONVERT_OPT_INVT					0x00001000
#define PI_BC_CONVERT_OPT_xxxx					0x0000f000

//flags for "flags" in ::read_rgb()
#define PI_BC_READRGB_LSB_IS_RED				0x00000001
#define PI_BC_READRGB_ALPHA_NORM				0x00000000
#define PI_BC_READRGB_ALPHA_INVT				0x00000002
#define PI_BC_READRGB_ALPHA_NONE				0x00000006
#define PI_BC_READRGB_ALPHA_xxxx				0x00000006



//-----------------------------------------------------------------------------
//pi_MetaMarkup
//-----------------------------------------------------------------------------
#define PI_METAMARKUP_VERSION					0x00010002

#pragma pack(push)
#pragma pack(1)
	struct pi_METAMARKUP
	{
		unsigned long	version;
		unsigned long	unused_1;
	//8
		int (__cdecl*	append)(char* markup,int size,char* fragment);
		int (__cdecl*	unused)(char* markup,int size);
		int (__cdecl*	get_tag)(char* markup,char* tag,char* value,int size);
		int (__cdecl*	set_tag)(char* markup,int size,char* tag,char* value);
		int (__cdecl*	bpp_get)(char* markup,int* bpp);
		int (__cdecl*	bpp_set)(char* markup,int size,int bpp);
		int (__cdecl*	ppi_get)(char* markup,double* x,double* y);
		int (__cdecl*	ppi_set)(char* markup,int size,double x,double y);
		int (__cdecl*	hotspot_get)(char* markup,int* x,int* y);
		int (__cdecl*	hotspot_set)(char* markup,int size,int x,int y);
		int (__cdecl*	exif_get)(char* markup,void** exif,int* sz_exif);
		int (__cdecl*	exif_set)(char* markup,int sz_mk,void* exif,int sz_ex);
		int (__cdecl*	exif_loadXMP)(char* markup,int sz_mk,char* xmp_str);
	//60
		unsigned char	reserved_x[1024-60];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//pi_BasicUtils
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	#define PI_BASICUTILS_VERSION					0x00010001

	struct pi_BASICUTILS
	{
		unsigned long	version;
		unsigned long	unused_1;
	//8
		int (__cdecl*	lang_TranslateCore)(wchar_t* out,int size,wchar_t* in);
		int (__cdecl*	lang_SetWindowText)(HWND hwnd,wchar_t* str);
		int (__cdecl*	lang_TranslateWindow)(HWND hwnd);
	//20
		unsigned char	reserved_x[1024-20];//do not touch
	};
#pragma pack(pop)


//=============================================================================



/******************************************************************************
*******************************************************************************
*******************************************************************************

                       FastExternals(TM) CORE STRUCTS

*******************************************************************************
*******************************************************************************
******************************************************************************/



//=============================================================================

//-----------------------------------------------------------------------------
//plg_INFO
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct plg_INFO
	{
		unsigned long	api_type;		// PLUGIN_APITYPE_* flags
		unsigned long	api_version;	// PLUGIN_INTERFACE_VERSION
		unsigned long	api_flags;		// do not touch
		unsigned long	api_extra;		// do not touch
	//16
		unsigned long	plg_version;	// plug-in's version number
		unsigned long	plg_options;	// plug-in's options
		unsigned long	plg_flags;		// do not touch
		unsigned long	plg_extra;		// do not touch
	//32
		wchar_t			plg_name[64];	// plug-in's name
		wchar_t			plg_author[64];	// plug-in's author 
	//288
		unsigned char	reserved_x[1024-288]; // do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//plg_DIALOG
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct plg_DIALOG
	{
		HWND			hwnd;
		unsigned long	dialog_id;
		unsigned long	param_1;
		unsigned long	param_2;
	//16
		unsigned char	reserved_1[256-16]; //do not touch
	//256
		pi_STATESTORE*	pi_StateStore;
		pi_METAMARKUP*	pi_MetaMarkup;
		pi_BASICQUERY*	pi_BasicQuery;
		pi_BASICIMAGE*	pi_BasicImage;
		pi_BASICCOLOR*	pi_BasicColor;
		pi_BASICUTILS*	pi_BasicUtils;
	//280
		unsigned char	reserved_x[1024-280];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//flt_IMAGE_L
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct flt_IMAGE_L
	{
		unsigned long	width;
		unsigned long	height;
		unsigned long	pitch;
		unsigned long*	lp_pix;
	//16
		wchar_t*		file;
		HWND			hwnd;
		int				(__cdecl* create)(flt_IMAGE_L* self);
		unsigned short	host_flags;
		unsigned char	img_type;
		unsigned char	img_subtype;
	//32
		unsigned long	layer_count;
		short			x_pos;
		short			y_pos;
		wchar_t			name[32];
		unsigned short	options;
		unsigned short	frame_delay;
		unsigned char	group_id;
		unsigned char	transparency;
		unsigned char	unused_2[18];
	//128
		unsigned char*	meta_data;
		int				(__cdecl* meta_write)(flt_IMAGE_L* self,char* type,int size);
	//136
		unsigned char	reserved_1[256-136-(4*4)];//do not touch
		int				host_response;
		unsigned char	reserved_2[3];//do not touch
		unsigned char	warnings;
		int				(__cdecl* set_err_info)(flt_IMAGE_L* self,wchar_t* info);
		unsigned long	timeout;
	//256
		pi_STATESTORE*	pi_StateStore;
		pi_METAMARKUP*	pi_MetaMarkup;
		pi_BASICQUERY*	pi_BasicQuery;
		pi_BASICIMAGE*	pi_BasicImage;
		pi_BASICCOLOR*	pi_BasicColor;
		pi_BASICUTILS*	pi_BasicUtils;
	//280
		unsigned char	reserved_x[1024-280];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//flt_CHECK_L
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct flt_CHECK_L
	{
		wchar_t			extension[32];
		unsigned char	hdr_data[32];
		unsigned long	hdr_size;
		unsigned char	reserved_x[128-4-100];//do not touch
		unsigned char	level_image;
		unsigned char	level_layer;
		unsigned char	level_other;
		unsigned char	support_flags;
	};
#pragma pack(pop)

#define SUPPORT_FLAG_IS_BRIDGE			0x00000001
#define SUPPORT_FLAG_DEMOTE_ME			0x00000002



//-----------------------------------------------------------------------------
//flt_IMAGE_S
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct flt_IMAGE_S
	{
		unsigned long	width;
		unsigned long	height;
		unsigned long	pitch;
		unsigned long*	lp_pix;
	//16
		wchar_t*		file;
		HWND			hwnd;
		int				(__cdecl* select)(flt_IMAGE_S* self,int layer_id);
		unsigned short	host_flags;
		unsigned char	img_type;
		unsigned char	img_subtype;
	//32
		unsigned long	layer_count;
		short			x_pos;
		short			y_pos;
		wchar_t			name[32];
		unsigned short	options;
		unsigned short	frame_delay;
		unsigned char	group_id;
		unsigned char	transparency;
		unsigned char	unused_2[18];
	//128
		unsigned char*	meta_data;
		int				(__cdecl* meta_read)(flt_IMAGE_S* self,char* type);
	//136
		unsigned char	reserved_1[256-136-(4*4)];//do not touch
		int				host_response;
		unsigned char	reserved_2[3];//do not touch
		unsigned char	warnings;
		int				(__cdecl* set_err_info)(flt_IMAGE_S* self,wchar_t* info);
		unsigned long	timeout;
	//256
		pi_STATESTORE*	pi_StateStore;
		pi_METAMARKUP*	pi_MetaMarkup;
		pi_BASICQUERY*	pi_BasicQuery;
		pi_BASICIMAGE*	pi_BasicImage;
		pi_BASICCOLOR*	pi_BasicColor;
		pi_BASICUTILS*	pi_BasicUtils;
	//280
		unsigned char	reserved_x[1024-280];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//flt_CHECK_S
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct flt_CHECK_S
	{
		wchar_t			extension[32];
		unsigned char	reserved_x[128-4-64];//do not touch
		unsigned char	level_image;
		unsigned char	level_layer;
		unsigned char	level_other;
		unsigned char	support_flags;
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//flt_IMAGE_X
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct flt_IMAGE_X 
	{
		wchar_t*		file;
		HWND			hwnd;
		unsigned long	xform_id;
		unsigned long	xform_id2;
	//16
		unsigned char	reserved_1[256-16-(4*4)];//do not touch
		int				host_response;
		unsigned char	reserved_2[3];//do not touch
		unsigned char	unused_1;
		int				(__cdecl* set_err_info)(flt_IMAGE_X* self,wchar_t* info);
		unsigned long	timeout;
	//256
		pi_STATESTORE*	pi_StateStore;
		pi_METAMARKUP*	pi_MetaMarkup;
		pi_BASICQUERY*	pi_BasicQuery;
		pi_BASICIMAGE*	pi_BasicImage;
		pi_BASICCOLOR*	pi_BasicColor;
		pi_BASICUTILS*	pi_BasicUtils;
	//280
		unsigned char	reserved_x[1024-280];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//dev_IMAGE_R
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct dev_IMAGE_R
	{
		unsigned long	width;
		unsigned long	height;
		unsigned long	pitch;
		unsigned long*	lp_pix;
	//16
		unsigned long	unused_1;
		HWND			hwnd;
		int				(__cdecl* create)(dev_IMAGE_R* self);
		unsigned short	host_flags;
		unsigned char	img_type;
		unsigned char	img_subtype;
	//32
		unsigned long	layer_count;
		short			x_pos;
		short			y_pos;
		wchar_t			name[32];
		unsigned short	options;
		unsigned short	frame_delay;
		unsigned char	group_id;
		unsigned char	transparency;
		unsigned char	unused_3[18];
	//128
		unsigned char*	meta_data;
		int				(__cdecl* meta_write)(dev_IMAGE_R* self,char* type,int size);
	//136
		unsigned char	reserved_1[256-136-(4*4)];//do not touch
		int				host_response;
		unsigned char	reserved_2[3];//do not touch
		unsigned char	warnings;
		int				(__cdecl* set_err_info)(dev_IMAGE_R* self,wchar_t* info);
		unsigned long	timeout;
	//256
		pi_STATESTORE*	pi_StateStore;
		pi_METAMARKUP*	pi_MetaMarkup;
		pi_BASICQUERY*	pi_BasicQuery;
		pi_BASICIMAGE*	pi_BasicImage;
		pi_BASICCOLOR*	pi_BasicColor;
		pi_BASICUTILS*	pi_BasicUtils;
	//280
		unsigned char	reserved_x[1024-280];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//dev_IMAGE_S
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct dev_IMAGE_S
	{
		unsigned long	width;
		unsigned long	height;
		unsigned long	pitch;
		unsigned long*	lp_pix;
	//16
		unsigned long	unused_1;
		HWND			hwnd;
		int				(__cdecl* select)(dev_IMAGE_S* self,int layer_id);
		unsigned short	host_flags;
		unsigned char	img_type;
		unsigned char	img_subtype;
	//32
		unsigned long	layer_count;
		short			x_pos;
		short			y_pos;
		wchar_t			name[32];
		unsigned short	options;
		unsigned short	frame_delay;
		unsigned char	group_id;
		unsigned char	transparency;
		unsigned char	unused_3[18];
	//128
		unsigned char*	meta_data;
		int				(__cdecl* meta_read)(dev_IMAGE_S* self,char* type);
	//136
		unsigned char	reserved_1[256-136-(4*4)];//do not touch
		int				host_response;
		unsigned char	reserved_2[3];//do not touch
		unsigned char	warnings;
		int				(__cdecl* set_err_info)(dev_IMAGE_S* self,wchar_t* info);
		unsigned long	timeout;
	//256
		pi_STATESTORE*	pi_StateStore;
		pi_METAMARKUP*	pi_MetaMarkup;
		pi_BASICQUERY*	pi_BasicQuery;
		pi_BASICIMAGE*	pi_BasicImage;
		pi_BASICCOLOR*	pi_BasicColor;
		pi_BASICUTILS*	pi_BasicUtils;
	//280
		unsigned char	reserved_x[1024-280];//do not touch
	};
#pragma pack(pop)



//-----------------------------------------------------------------------------
//efx_IMAGE_T
//-----------------------------------------------------------------------------
#pragma pack(push)
#pragma pack(1)
	struct efx_IMAGE_T
	{
		unsigned long	width;
		unsigned long	height;
		unsigned long	pitch;
		unsigned long*	lp_pix;
	//16
		HWND			hwnd;
		unsigned long	flags;
	//24
		int				(__cdecl* lock   )(void);
		int				(__cdecl* unlock )(void);
		int				(__cdecl* load   )(void);
		int				(__cdecl* realloc)(void);
		int				(__cdecl* refresh)(void);
		int				(__cdecl* progress)(int done,int total);
	//48
		unsigned long	color_1;
		unsigned long	color_2;
		unsigned long	unused_1;
		unsigned long	unused_2;
	//64
		unsigned char*	(__cdecl* mem_alloc)(unsigned long size);
		unsigned char*	(__cdecl* mem_resize)(unsigned char* mem,unsigned long size);
		int				(__cdecl* mem_free)(unsigned char* mem);
		unsigned long	unused_3;
	//80
		unsigned char*	meta_data;
		unsigned long	(__cdecl* meta_query)(char* type,unsigned long size);
	//88
		int				(__cdecl* dock)(HWND hwnd);
		unsigned long	unused_4;
	//96
		unsigned char	reserved_1[252-96];//do not touch
		unsigned long	unused_5;
	//256
		pi_STATESTORE*	pi_StateStore;
		pi_METAMARKUP*	pi_MetaMarkup;
		pi_BASICQUERY*	pi_BasicQuery;
		pi_BASICIMAGE*	pi_BasicImage;
		pi_BASICCOLOR*	pi_BasicColor;
		pi_BASICUTILS*	pi_BasicUtils;
	//280
		unsigned char	reserved_x[1024-280];//do not touch
	};
#pragma pack(pop)


//=============================================================================



/******************************************************************************
*******************************************************************************
*******************************************************************************

                 FastExternals(TM) CORE FUNCTION PROTOTYPES

*******************************************************************************
*******************************************************************************
******************************************************************************/



//=============================================================================

//-----------------------------------------------------------------------------
//support functions
typedef int (__cdecl* FUNC_FLT_CREATE)(void*);

//-----------------------------------------------------------------------------
//all plugins
typedef int (__cdecl* FUNC_PLG_INFO  )(struct plg_INFO*  );
typedef int (__cdecl* FUNC_PLG_DIALOG)(struct plg_DIALOG*);

//-----------------------------------------------------------------------------
//new filter layer (FastExternals)
typedef int (__cdecl* FUNC_FLT_LOADIMAGE)(struct flt_IMAGE_L*);
typedef int (__cdecl* FUNC_FLT_SAVEIMAGE)(struct flt_IMAGE_S*);
typedef int (__cdecl* FUNC_FLT_CHECKLOAD)(struct flt_CHECK_L*);
typedef int (__cdecl* FUNC_FLT_CHECKSAVE)(struct flt_CHECK_S*);
typedef int (__cdecl* FUNC_FLT_XFRMIMAGE)(struct flt_IMAGE_X*);

//-----------------------------------------------------------------------------
//new device layer (FastExternals)
typedef int (__cdecl* FUNC_DEV_READIMAGE)(struct dev_IMAGE_R*);
typedef int (__cdecl* FUNC_DEV_SENDIMAGE)(struct dev_IMAGE_S*);

//-----------------------------------------------------------------------------
//new effects layer (FastExternals)
typedef int (__cdecl* FUNC_EFX_DO_EFFECT)(struct efx_IMAGE_T*);

//=============================================================================



//=============================================================================
//=============================================================================
